const colorLegend = (selection, props) => {
    const {
        colorScale,
        spacing,
        textOffset
    } = props;

    const groups = selection.selectAll('g')
        .data([50,52,54,56,58,60]);
    const groupsEnter = groups
        .enter().append('g')
        .attr('class', 'tick');
    groupsEnter
        .merge(groups)
        .attr('transform', (d, i) =>
            `translate(0, ${i * spacing})`
        );
    groups.exit().remove();

    groupsEnter.append('circle')
        .merge(groups.select('circle'))
        .attr('r', 15)
        .attr('class','circleColorLeyend')
        .attr('fill',colorScale);

    groupsEnter.append('text')
        .merge(groups.select('text'))
        .text(d => d + '%')
        .attr('class','textLeyend')
        .attr('dy', '0.32em')
        .attr('x', textOffset);
}
function sizeLegend (selection, props) {
    const {
        sizeScale,
        spacing,
        textOffset,
        numTicks,
    } = props;

    const ticks = sizeScale.ticks(numTicks)
        .filter(d => d !== 0)
        .reverse();

    const groups = selection.selectAll('g').data(ticks);
    const groupsEnter = groups
        .enter().append('g')
        .attr('class', 'tick');
    groupsEnter
        .merge(groups)
        .attr('transform', (d, i) =>
            `translate(0, ${i * spacing})`
        );
    groups.exit().remove();

    groupsEnter.append('circle')
        .merge(groups.select('circle'))
        .attr('r', sizeScale)
        .attr('class','circleLeyend');

    groupsEnter.append('text')
        .merge(groups.select('text'))
        .text(d => d)
        .attr('class','textLeyend')
        .attr('x', d => sizeScale(d) + textOffset);

}

const svg = d3.select('svg');
const projection = d3.geoMercator().center([-4,40.3]); //centrar el mapa
projection.scale(2700); //cambiar la escala del mapa

const pathGenerator = d3.geoPath().projection(projection);

//cargar el mapa con las siluetas de los paises en topoJSON y el csv con datos covid
Promise.all([d3.json('https://unpkg.com/es-atlas@0.5.0/es/municipalities.json'),d3.csv('data.csv')])
    .then(([topoJSONdata,csvData]) => {

        const rowById = {};
        //convertir datos Covid en array de objetos
        csvData.forEach(d=> {
            d.porcentaje = +d.porcentaje * 100; //multiplicamos el porcentaje por 100
            d.casos = +d.casos;
            rowById[d.id] = d;
        });

        const provinces = topojson.feature(topoJSONdata, topoJSONdata.objects.provinces);

        //hacer que se pueda acceder a la información del csv mediante el nombre de provincia del fichero TOPOjson
        provinces.features.forEach(d=> {
            Object.assign(d.properties, rowById[d.id])
        });
  
                      //DIBUJAR PROVINCIAS
                svg.selectAll('path').data(provinces.features) //crear un objeto svg para cada provincia
                    .enter().append('path')
                    .attr('class', 'province')
                    .attr('d', pathGenerator)

          function dibujar(csvData){
                //DIBUJAR CIRCULOS
                const sizeScale = d3.scaleSqrt() //escala que convierte los datos a una escala más pequeña para representarlos
                    .domain([0,d3.max(csvData,d => d.casos)]) //intervalo datos reales
                    .range([0,40]); //intervalo que vemos por pantalla

                var colorScale = d3.scaleSequential(d3.interpolateRdYlBu) //escala de color
                    .domain([50,60]);

                svg.selectAll('circle').data(provinces.features) //crear un objeto svg para cada provincia
                    .enter().append('circle')
                    .attr('class','circles')
                    .attr('cx', d => projection(d3.geoCentroid(d))[0]) //poner circulos en el centro de cada pais
                    .attr('cy', d => projection(d3.geoCentroid(d))[1])
                    .attr('r',d => sizeScale(d.properties.casos)) //le ponemos el ancho de la población que tenga en Linear Scale
                    .attr('fill',d=> colorScale(d.properties.porcentaje))
                    .append('title') //añadir cuadro de texto que muestra info de cada provincia
                    .text(d => d.properties.provincia + '\nCasos: '+ (d.properties.casos) + '\nPorcentaje: '+ (d.properties.porcentaje));

                //AÑADIR LEYENDAS
                svg.append('g') //añadir la leyenda de tamaños
                    .attr('transform', `translate(50,200)`) //colocar en la posicion
                    .call(sizeLegend, {
                        sizeScale,
                        spacing: 60,
                        textOffset: 4,
                        numTicks: 5,
                    });

                svg.append('g') //añadir la leyenda de colores
                    .attr('transform', `translate(160,200)`) //colocar en la posicion
                    .call(colorLegend, {
                        colorScale,
                        spacing: 40,
                        textOffset: 20,
                    });
          } 

				// Manejador de eventos
    		d3.select("#ctrl_mes").on("change", function(evt) { actualizar(csvData); });
 				actualizar(csvData);
     		function actualizar(csvData) {
       			csvData = csvData.filter(d => (d.mes) == 'junio');
 				 		dibujar(csvData)
    		}
        
   